/*
 *	CddaPlayer.java
 *
 *	This file is part of jsresources.org
 */

/*
 * Copyright (c) 2001 by Matthias Pfisterer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 |<---            this code is formatted to fit into 80 columns             --->|
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;

import org.tritonus.lowlevel.cdda.CddaMidLevel;
import org.tritonus.lowlevel.cdda.CddaUtils;

public class CddaPlayer {

	public static void main(String[] args) throws Exception {
		String strDrive = "/dev/cdrom1";

		fetchCDToc(strDrive);
		playTrack(strDrive);
	}

	private static void fetchCDToc(String cdDrive) throws Exception {
		CddaMidLevel cddaMidLevel = CddaUtils.getCddaMidLevel();
		InputStream in = cddaMidLevel.getTocAsXml(cdDrive);
		BufferedReader aReader = new BufferedReader(new InputStreamReader(in));
		String line = null;
		while ((line = aReader.readLine()) != null) {
			System.out.println(line);
		}
	}

	private static void playTrack(String strDrive) throws Exception {
		// obtain audio stream
		CddaMidLevel cddaMidLevel = CddaUtils.getCddaMidLevel();
		AudioInputStream audioIn = cddaMidLevel.getTrack(strDrive, 1);

		// obtain audio information to create the appropriate dat line (for
		// audio output
		AudioFormat audioFormat = audioIn.getFormat();
		Line.Info info = new DataLine.Info(SourceDataLine.class, audioFormat);

		// create a DataLine to play audio
		SourceDataLine dataLine = (SourceDataLine) AudioSystem.getLine(info);
		dataLine.open();
		dataLine.start();
		
		// feed audio bytes to the data line
		int nBytesRead = 0;
		byte[] abData = new byte[128000];
		while ((nBytesRead = audioIn.read(abData, 0, abData.length)) > -1) {
			int nBytesWritten = dataLine.write(abData, 0, nBytesRead);
		}
		
		// done, close the data line and streams
		audioIn.close();
		dataLine.flush();
		dataLine.close();
	}

}
